#include "basics2d.hh"
#include "core/type.hh"
#include "morpho/erosion.hh"
#include "morpho/dilation.hh"
#include "morpho/opening.hh"
#include "morpho/closing.hh"
#include "morpho/laplacian.hh"
#include "morpho/gradient.hh"
#include "morpho/top_hat.hh"
#include "morpho/hit_or_miss.hh"
#include "morpho/watershed.hh"

using namespace oln;

int main()
{
  image2d<int_u8> lena;

  read_pnm(lena, "../img/lena.pgm");

  save_pnm(morpho::erosion(lena,  win_c8p()), "lena-ero.pgm");
  save_pnm(morpho::dilation(lena, win_c8p()), "lena-dil.pgm");
  save_pnm(morpho::opening(lena,  win_c8p()), "lena-ope.pgm");
  save_pnm(morpho::closing(lena,  win_c8p()), "lena-clo.pgm");
  save_pnm(morpho::laplacian(convert::force<int_u8>(), lena, win_c8p()),
	   "lena-lap.pgm");

  save_pnm(morpho::beucher_gradient(lena, win_c8p()), "lena-beg.pgm");
  save_pnm(morpho::internal_gradient(lena, win_c8p()), "lena-ing.pgm");
  save_pnm(morpho::external_gradient(lena, win_c8p()), "lena-exb.pgm");

  save_pnm(morpho::white_top_hat(lena, win_c8p()), "lena-wth.pgm");
  save_pnm(morpho::black_top_hat(lena, win_c8p()), "lena-bth.pgm");
  save_pnm(morpho::self_complementary_top_hat(lena, win_c8p()),
	   "lena-sth.pgm");
  save_pnm(morpho::top_hat_contrast_op(convert::bound<int_u8>(),
				       lena, win_c8p()), "lena-thc.pgm");

  window2d mywin;
  mywin
    .add(-3,-2).add(-3,-1).add(-3,0).add(-3,1).add(-3,2)
    .add(-2,-1).add(-2,0).add(-2,1)
    .add(-1,0);
  save_pnm(morpho::closing(lena, mywin), "lena-myw.pgm");

  window2d mywin2 = - mywin;

  save_pnm(morpho::hit_or_miss(lena, mywin, mywin2), "lena-hom.pgm");

  neighborhood2d n = neighb_c8 ();
  save_pnm(morpho::watershed_seg<int_u8>(lena, n), "lena-wss.pgm");

  image2d<int_u8> markers(lena.size());
  image2d<int_u8>::iter i(markers);
  for_all(i) markers[i] = 0;

  for (int i = 0; i < 10; ++i)
    for (int j = 0; j < 10; ++j)
      {
	markers(477 + i, 67 + j) = 72;
	markers(100 + i, 200 + j) = 20;
	markers(50 + i, 50 + j) = 60;
	markers(100 + i, 500 + j) = 200;
	markers(255 + i, 355 + j) = 120;
      }

  save_pnm
    (morpho::watershed_seg_or
     (morpho::closing
      (morpho::beucher_gradient(lena, win_c8p()), win_c8p()), markers,
      neighb_c8()),
     "lena-wor.pgm");
}
