// Copyright 2001  EPITA Research and Development Laboratory
//
// This file is part of the Olena Library.  This library is free
// software; you can redistribute it and/or modify it under the terms
// of the GNU General Public License version 2 as published by the
// Free Software Foundation.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this library; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// As a special exception, you may use this file as part of a free
// software library without restriction.  Specifically, if other files
// instantiate templates or use macros or inline functions from this
// file, or you compile this file and link it with other files to
// produce an executable, this file does not by itself cause the
// resulting executable to be covered by the GNU General Public
// License.  This exception does not however invalidate any other
// reasons why the executable file might be covered by the GNU General
// Public License.

#ifndef OLENA_CORE_IMAGE1D_SIZE_HH
# define OLENA_CORE_IMAGE1D_SIZE_HH

# include "core/image_size.hh"
# include "core/coord.hh"

namespace oln {

  class image1d_size : public image_size< image1d_size >
  {
  public:

    typedef type::bottom inferior;

    enum { dim = 1 };

    image1d_size(coord ncols) :
    _ncols(ncols)
    {
    }

    coord ncols() const
    {
      invariant(_ncols > 0);
      return _ncols;
    }

    bool operator==(const image1d_size& size) const
    {
      return this->_ncols == size.ncols();
    }

    bool operator!=(const image1d_size& size) const
    {
      return this->_ncols != size.ncols();
    }

    static std::string name() { return "image1d_size"; }

  private:

    coord _ncols;
  };

  _ImageSizeForDim(1, image1d_size);

} // end of oln

#endif // ! OLENA_CORE_IMAGE1D_SIZE_HH
